classdef GaussNewton < falcon.parest.optimization.Optimizer & falcon.parest.optimization.ActiveSet
    % FALCON.PAREST.OPTIMIZATION.GAUSSNEWTON implementation of Gauss-Newton
    %   optimization algorithm
    %  
    %  	<Description>
    %  	This class implements the optimization step algorithm according to
    %  	a pure Gauss-Newton scheme. Local divergence is avoided by using a
    %  	heuristic step-halving approach to guarantuee a decrease in the
    %  	cost function.
    %  
    %  	<ChangeLog>
    %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % maximum number of minor interations
        MaxMinorIter
        % logical index vector for parameters that are free
        FreeVariables
        % logical index vector for parameters that are fixed
        FixedVariables
    end

    methods
        function [J, Success, MinorIteration] = step(obj, p, J, dJ, ddJ, Experiments, CostFunction)
        % STEP compute one optimization step
        %  
        %  	<Syntax>
        %  	[ J, Success, MinorIteration ] = step( obj, p, J, dJ, ddJ, Experiments, CostFunction )
        %  
        %  	<Description>
        %  	compute one optimization step according to the Gauss-Newton
        %  	Algorithm with a step halving strategy in order to prevent
        %  	divergence
        %  
        %  	<Inputs>
        %  	>obj: the GaussNewton object
        %  	>p: vector of falcon.parest.Parameter object holding the
        %  	parameter's characteristics
        %  	>J: current cost function value
        %  	>dJ: current gradient value
        %  	>ddJ: current hessian value
        %  	>Experiments: array of falcon.parest.Experiment objects
        %  	holding the experiment data
        %  	>CostFunction: falcon.parest.costfunction.CostFunction
        %  	object for the computation of updated cost function values
        %  
        %  	<Outputs>
        %  	>J: updated cost function value
        %  	>Success: flag indicating if step was successfule
        %  	>MinorIteration: number of minor iterations, i.e. halving
        %  	steps that had to be taken in order to achieve a cost
        %  	function reduction
        %  
        %  	<ChangeLog>
        %  	2017-01-24 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-26 - Ch. Goettlicher: udpate to be compatible with
        %   Bayesian estimation
        end

        function obj = GaussNewton()
        % FALCON.PAREST.OPTIMIZATION.GAUSSNEWTON implementation of Gauss-Newton
        %   optimization algorithm
        %  
        %  	<Description>
        %  	This class implements the optimization step algorithm according to
        %  	a pure Gauss-Newton scheme. Local divergence is avoided by using a
        %  	heuristic step-halving approach to guarantuee a decrease in the
        %  	cost function.
        %  
        %  	<ChangeLog>
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function setFixedIntermediate(obj, FixedIntermediate)
        % falcon.parest.optimization.ActiveSet/setFixedIntermediate is a function.
        %   setFixedIntermediate(obj, FixedIntermediate)
        end

        function initializeActiveSet(obj, p)
        % INITIALIZEACTIVESET initialize active set algortihm
        %  
        %  	<Syntax>
        %  	initializeActiveSet( obj, p )
        %  
        %  	<Description>
        %  	initialize active set algorithm by comparing the free
        %  	parameter values to their bounds. Violating parameters are
        %  	set to the respective bounds, and their indices stored in the
        %  	respective logical index vector
        %  
        %  	<Inputs>
        %  	>obj: the ActiveSet object
        %  	>p: vector of falcon.parest.Parameter object holding the
        %  	parameter's characteristics
        %  
        %  	<ChangeLog>
        %  	2017-01-24 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

    end

end