classdef LevenbergMarquardt < falcon.parest.optimization.Optimizer & falcon.parest.optimization.ActiveSet
    % FALCON.PAREST.OPTIMIZATION.LEVENBERGMARQUARDT implementation of
    %   Levenberg-Marquardt optimization algorithm
    %  
    %  	<Description>
    %  	This class implements an optimization step algorithm according to
    %  	the Levenberg-Marquardt scheme with damping parameters lambda, and
    %  	nu
    %  
    %  	<ChangeLog>
    %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % Levenberg Marquardt parameter lambda
        lambda
        % Levenberg Marquardt parameter nu
        nu
        % maximum number of minor interations
        MaxMinorIter
        % logical index vector for parameters that are free
        FreeVariables
        % logical index vector for parameters that are fixed
        FixedVariables
    end

    methods
        function [J, Success, MinorIteration] = step(obj, p, J, dJ, ddJ, Experiments, CostFunction)
        % STEP compute one optimization step
        %  
        %  	<Syntax>
        %  	[ J, Success, MinorIteration ] = step( obj, p, J, dJ, ddJ, Experiments, CostFunction )
        %  
        %  	<Description>
        %  	compute one optimization step according to the
        %  	Levenberg-Marquardt Algorithm with scaled Hessian and
        %  	Gradient.
        %  
        %  	<Inputs>
        %  	>obj: 
        %  	>p: vector of falcon.parest.Parameter object holding the
        %  	parameter's characteristics
        %  	>J: current cost function value
        %  	>dJ: current gradient value
        %  	>ddJ: current hessian value
        %  	>Experiments: array of falcon.parest.Experiment objects
        %  	holding the experiment data
        %  	>CostFunction: falcon.parest.costfunction.CostFunction
        %  	object for the computation of updated cost function values
        %  
        %  	<Outputs>
        %  	>J: updated cost function value
        %  	>Success: flag indicating if step was successfule
        %  	>MinorIteration: number of minor iterations, i.e.
        %  	Levenberg-Marquard steps that had to be taken in order to
        %  	achieve a cost function reduction
        %  
        %  	<ChangeLog>
        %  	2017-01-24 - Ch. Goettlicher: initial design
        %  	2017-10-26 - Ch. Goettlicher: changed to conventional LM
        %                scaling
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-26 - Ch. Goettlicher: bugfix with active set,
        %   boundaries were wrongly considered previously
        end

        function obj = LevenbergMarquardt()
        % FALCON.PAREST.OPTIMIZATION.LEVENBERGMARQUARDT implementation of
        %   Levenberg-Marquardt optimization algorithm
        %  
        %  	<Description>
        %  	This class implements an optimization step algorithm according to
        %  	the Levenberg-Marquardt scheme with damping parameters lambda, and
        %  	nu
        %  
        %  	<ChangeLog>
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function setFixedIntermediate(obj, FixedIntermediate)
        % falcon.parest.optimization.ActiveSet/setFixedIntermediate is a function.
        %   setFixedIntermediate(obj, FixedIntermediate)
        end

        function initializeActiveSet(obj, p)
        % INITIALIZEACTIVESET initialize active set algortihm
        %  
        %  	<Syntax>
        %  	initializeActiveSet( obj, p )
        %  
        %  	<Description>
        %  	initialize active set algorithm by comparing the free
        %  	parameter values to their bounds. Violating parameters are
        %  	set to the respective bounds, and their indices stored in the
        %  	respective logical index vector
        %  
        %  	<Inputs>
        %  	>obj: the ActiveSet object
        %  	>p: vector of falcon.parest.Parameter object holding the
        %  	parameter's characteristics
        %  
        %  	<ChangeLog>
        %  	2017-01-24 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

    end

end