classdef Grid < falcon.core.Handle & falcon.core.HasToStruct
    % FALCON.TRIM.CORE.GRID grid object to store trim/lin results
    %  
    %  	<Description>
    %  	this grid can hold any type of information in its "Value" property,
    %  	it is the main mechanism to store data. Most of its capabilities
    %  	however, are related to dynamically increasing the size of the grid
    %  	in higher dimension, so that trimming and linearizing is easily
    %  	possible in dimensions > 3. 
    %  
    %  	<ChangeLog>
    %  	2017-12-20 - Ch. Goettlicher: final commenting

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
    end

    methods
        function addDimension(obj, array, GenerateFlag)
        % ADDDIMENSION add dimension to ND value grid
        %  
        %  	<Syntax>
        %  	addDimension( obj, array, GenerateFlag )
        %  
        %  	<Description>
        %  	adds a new dimension to the value grid. new dimensions are
        %  	appended in the end. Depending on the GenerateFlag, the
        %  	existing values are duplicated using repmat, or the input
        %  	array is used to span a new grid
        %  
        %  	<Inputs>
        %  	>obj: 
        %  	>array: array, which is used to expand the value grid.
        %  	Either only its size is used (GenerateFlag = false), or it is
        %  	used, to build up a new ND-array (GenerateFlag = true, if
        %  	this is a breakpoint variable)
        %  	>GenerateFlag: decides if old values are to be duplicated,
        %  	or new Grid is to be generated
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function addBreakpoints(obj, array, name)
        % ADDBREAKPOINTS add a breakpoint name-value pair
        %  
        %  	<Syntax>
        %  	addBreakpoints( obj, array, name )
        %  
        %  	<Description>
        %  	store breakpoint information with name and array in pts
        %  	property
        %  
        %  	<Inputs>
        %  	>obj: the Grid object
        %  	>array: numerical values of breakpoitns
        %  	>name: name of breakpoint vector
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function setTrimmed(obj, trimmed, idx)
        % SETTRIMMED set trimmed property at index idx
        %  
        %  	<Syntax>
        %  	setTrimmed( obj, trimmed, idx )
        %  
        %  	<Description>
        %  	set the trimmed property at index idx
        %  
        %  	<Inputs>
        %  	>obj: the Grid object
        %  	>trimmed: trimmed flag to set
        %  	>idx: linear idx in ND grid, where trimmed flag is to be
        %  	set
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function Size = getSize(obj)
        % GETSIZE get size of value grid
        %  
        %  	<Syntax>
        %  	[ Size ] = getSize( obj )
        %  
        %  	<Description>
        %  	get size of value grid
        %  
        %  	<Inputs>
        %  	>obj: the Grid object
        %  
        %  	<Outputs>
        %  	>Size: size vector for value grid
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function setObjectPropertyValue(obj, property, value, idx)
        % SETOBJECTPROPERTYVALUE if value is an object, set specific
        %   property
        %  
        %  	<Syntax>
        %  	setObjectPropertyValue( obj, property, value, idx )
        %  
        %  	<Description>
        %  	if values hold objects, set property of that object at
        %  	specific idx
        %  
        %  	<Inputs>
        %  	>obj: the Grid object
        %  	>property: property to be set
        %  	>value: value to set
        %  	>idx: linear index of object in ND grid to be modified
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function setValue(obj, value, idx)
        % SETVALUE set value in grid
        %  
        %  	<Syntax>
        %  	setValue( obj, value, idx )
        %  
        %  	<Description>
        %  	store a value in the grid, at a certain linear index
        %  
        %  	<Inputs>
        %  	>obj: the Grid object
        %  	>value: value to be set
        %  	>idx: linear idx, where to store value in grid
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function [value, trimmed] = getSubGrid(obj, varargin)
        % GETSUBGRID get values corresponding to indices
        %  
        %  	<Syntax>
        %  	[ value, trimmed ] = getSubGrid( obj, dims )
        %  
        %  	<Description>
        %  	get a sub-grid of this grid via dimensional indexing; the
        %  	return value is a cell array
        %  
        %  	<Inputs>
        %  	>obj: the Grid object
        %  	>dims: dimension elements to be extracted
        %  	>idx: indices of the subgrid
        %  
        %  	<Outputs>
        %  	>value: extracted values
        %  	>trimmed: cell array containing the 'trimmed' flag of the
        %  	grid
        %  
        %  	<ChangeLog>
        %  	2017-03-14 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function value = getSerializedValue(obj, idx)
        % GETSERIALIZEDVALUE get values corresponding to linear idx
        %  
        %  	<Syntax>
        %  	[ value ] = getSerializedValue( obj, idx )
        %  
        %  	<Description>
        %  	get the values from the grid in serialized fashion via the
        %  	linear index 'idx'
        %  
        %  	<Inputs>
        %  	>obj: the Grid object
        %  	>idx: linear indices to extract from the grid
        %  
        %  	<Outputs>
        %  	>value: extracted values
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function [XValues, YValues, PlotValues, XValueNames, YValueNames, Trimmed] = getValueForPlot(obj, varargin)
        % GETVALUEFORPLOT extract subset of grid values for plots
        %  
        %   <Syntax>
        %   [XValues, YValues, PlotValues, XValueNames, YValueNames] = getValueForPlot(obj, XValues, 'YValues', YValues)
        %   [XValues, YValues, PlotValues, XValueNames, YValueNames] = getValueForPlot(obj, XValues, 'RemainingBreakpoints', RemainingBreakpoints)
        %  
        %  	<Description>
        %  	extract values from grid to be plotted. Values are
        %  	extracted based on the XValue name (which will serve as
        %  	XValues in plot) and possibly YValue names. All other
        %  	breakpoints have to be defined via RemainingBreakpoints
        %  	(which are then stored in the Unmatched field of the input
        %  	parser. XValues, YValues, PlotValues and corresponding
        %  	names are returned.
        %  
        %  	<Inputs>
        %  	>XValues: name of breakpoint vector to be used as x-values
        %  
        %  	<NameValue>
        %  	>obj: the Grid object
        %  	>YValues: if this is value is included, a 3D plot is
        %  	generated using XValue, YValues and the grid values on the
        %  	z-axis
        %  	>RemainingBreakpoints: names of remaining breakpoints to
        %  	fully define a point in the underlying ND-grid
        %  
        %  	<Outputs>
        %  	>XValues: x value breakpoint vector
        %  	>YValues: y value breakpoint vector
        %  	>PlotValues: plot values from grid
        %  	>XValueNames: x value names
        %  	>YValueNames: y value names
        %  	>Trimmed: flag indicating if the points were trimmed
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function [SelectIdx, pts] = getValueByBreakpoints(obj, varargin)
        % GETVALUEBYBREAKPOINTS get indices of certain breakpoint
        %  
        %   <Syntax>
        %   [SelectIdx, pts] = obj.getValueByBreakpoints('BreakpointName1', BreakpointValues1, ...)
        %  
        %  	<Description>
        %  	extract the indices of the underlying grid's values by
        %  	defining desired values for the breakpoints of this grid.
        %  	The desired breakpoints are entered as name-value pairs,
        %  	where the names have to match the breakpoint names of the
        %  	grid. The values can be any array of numeric values, where
        %  	the closest match in the underlying grid's breakpoints are
        %  	returned (no interpolation is performed).
        %  	All breakpoints, that are present in the grid have to be
        %  	specified, by alternating inputs of names and values.
        %   e.g.
        %       Idx = obj.getValueByBreakpoints('V', [10:5:20], 'h', 1000)
        %   returns the values for velocities V = 10, 15, 20, and
        %   altitidue h = 1000 (or the respective closest available
        %   match)
        %   This can then be used with the getSubGrid method to get the
        %   actual values
        %       val = obj.getSubGrid( Idx{:} )
        %  
        %  	<Inputs>
        %  	>obj: the Grid object
        %  	>BreakpointNameXX: name of one breakpoint to be considered
        %  	>BreakpointValuesXX: desired values of the respective
        %  	breakpoint
        %  
        %  	<Outputs>
        %  	>SelectIdx: cell of indices to be selected from the value
        %  	grid
        %  	>pts: struct array of breakpoints (names and values) that
        %  	are actually treated, i.e. that were matched to the
        %  	underlying grid.
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function s = ToStruct(obj)
        % TOSTRUCT output data in struct
        %  
        %  	<Syntax>
        %  	[ s ] = ToStruct( obj )
        %  
        %  	<Description>
        %  	store data as struct
        %  
        %  	<Inputs>
        %  	>obj: the Grid object
        %  
        %  	<Outputs>
        %  	>s: the struct
        %  
        %  	<ChangeLog>
        %  	2017-03-13 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = Grid()
        % GRID constructor
        %  
        %  	<Syntax>
        %  	[ obj ] = Grid( obj )
        %  
        %  	<Description>
        %  	create a grid, that holds trim or linearization information
        %  	in an ND-grid
        %  
        %  	<Outputs>
        %  	>obj: the newly created object
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = fromStruct(s)
        % FROMSTRUCT create objects from struct
        %  
        %  	<Syntax>
        %  	[ obj ] = falcon.trim.core.Grid.fromStruct( s )
        %  
        %  	<Description>
        %  	create an array of objects from struct information
        %  
        %  	<Inputs>
        %  	>s: struct containing the object information
        %  
        %  	<Outputs>
        %  	>obj: the newly created object array
        %  
        %  	<ChangeLog>
        %  	2017-03-14 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

    end

end