classdef TrimCost < falcon.core.Handle & falcon.trim.core.HasTrimProblem
    % FALCON.TRIM.CORE.TRIMCOST cost function for trim problems
    %  
    %  	<Description>
    %  	The trim problem is internally transformed into an optimization
    %  	problem of the form
    %       x_dot = f(x,u,p)
    %       x_dot(SteadyState) = f_steady(x,u,p)
    %  
    %     ---                                                   ---
    %     |  min_(x_free,u_free,p_free) f_steady.' * f_steady     |
    %     |     s.t.    y(x,u,p) = y_desired                      |
    %     |             lb < constraints( x,u,p ) < ub            |
    %     ---                                                   ---
    %  
    %   that is, the state derivatives of states that were declared
    %   'SteadyState' is driven to zero by minimizing above cost function,
    %   subject to output and more general constraints.
    %  
    %   This class then implements the necessary cost function computations
    %  
    %  	<ChangeLog>
    %  	2017-12-20 - Ch. Goettlicher: final commenting

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
    end

    methods
        function JacStructure = returnJacStructure(obj)
        % RETURNJACSTRUCTURE jacobian structure callback.
        %  
        %  	<Syntax>
        %  	[ JacStructure ] = returnJacStructure( obj )
        %  
        %  	<Description>
        %  	No structure is deterimed, jacobian is passed as full
        %  
        %  	<Inputs>
        %  	>obj: the TrimCost object
        %  
        %  	<Outputs>
        %  	>JacStructure: structure of jacobian as sparse matrix
        %  
        %  	<ChangeLog>
        %  	2016-11-22 - Ch. Goettlicher: initial version
        %  	2017-12-15 - Ch. Goettlicher: separated problem from solver
        %                and cost
        %  	2017-02-07 - Ch. Goettlicher: commenting
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function ConstrGrad = Constrgrad(obj, z)
        % CONSTRGRAD jacobian callback
        %  
        %  	<Syntax>
        %  	[ ConstrGrad ] = Constrgrad( obj, z )
        %  
        %  	<Description>
        %  	jacobian callback function using the model and constraint
        %  	derivatives
        %  
        %  	<Inputs>
        %  	>obj: the TrimCost object
        %  	>z: optimization vector, used to update the inputs to the
        %  	model
        %  
        %  	<Outputs>
        %  	>ConstrGrad: jacobian of constraints
        %  
        %  	<ChangeLog>
        %  	2016-11-22 - Ch. Goettlicher: initial version
        %  	2016-12-15 - Ch. Goettlicher: separated problem from solver
        %                and cost
        %  	2017-02-07 - Ch. Goettlicher: commenting
        %  	2017-03-08 - Ch. Goettlicher: update to new
        %                updateModelInputs version
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function Constr = ConstrFun(obj, z)
        % CONSTRFUN constraint function callback.
        %  
        %  	<Syntax>
        %  	[ Constr ] = ConstrFun( obj, z )
        %  
        %  	<Description>
        %  	Constraints are put together from output constraints (defined
        %  	via trim-outputs) and additional constraints
        %  
        %  	<Inputs>
        %  	>obj: the TrimCost object
        %  	>z: optimization vector, used to update the inputs to the
        %  	model
        %  
        %  	<Outputs>
        %  	>Constr: constraint violation
        %  
        %  	<ChangeLog>
        %  	2016-11-22 - Ch. Goettlicher: initial version
        %  	2016-12-15 - Ch. Goettlicher: separated problem from solver
        %                and cost
        %  	2017-02-07 - Ch. Goettlicher: commenting
        %  	2017-03-08 - Ch. Goettlicher: boundaries for all outputs
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function dJ = Jgrad(obj, z)
        % JGRAD gradient of cost function.
        %  
        %  	<Syntax>
        %  	[ dJ ] = Jgrad( obj, z )
        %  
        %  	<Description>
        %  	Analytic gradient of sum of squares cost together with
        %  	derivatives of the model
        %  
        %  	<Inputs>
        %  	>obj: the TrimCost object
        %  	>z: optimization vector, used to update the inputs to the
        %  	model
        %  
        %  	<Outputs>
        %  	>dJ: cost function gradient
        %  
        %  	<ChangeLog>
        %  	2016-11-22 - Ch. Goettlicher: initial version
        %  	2016-12-15 - Ch. Goettlicher: separated problem from solver
        %                and cost
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function J = Jfun(obj, z)
        % JFUN cost function callback
        %  
        %  	<Syntax>
        %  	[ J ] = Jfun( obj, z )
        %  
        %  	<Description>
        %  	cost-function is sum of squares of steady state
        %  	derivatives, which have to be zero for trim point
        %  
        %  	<Inputs>
        %  	>obj: the TrimCost object
        %  	>z: optimization vector, used to update the inputs to the
        %  	model
        %  
        %  	<Outputs>
        %  	>J: cost function value
        %  
        %  	<ChangeLog>
        %  	2016-11-22 - Ch. Goettlicher: initial version
        %  	2016-12-15 - Ch. Goettlicher: separated problem from solver
        %                and cost
        %  	2017-02-07 - Ch. Goettlicher: commenting
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function varargout = steadyStates(obj, z)
        % STEADYSTATES get values that are supposed to be steady state
        %  
        %  	<Syntax>
        %  	[ xd, jxd ] = steadyStates( obj, z )
        %  
        %  	<Description>
        %  	extract state values of the states are are considered steady.
        %  	if two output arguments are desired, output the respective
        %  	jacobian, too.
        %  
        %  	<Inputs>
        %  	>obj: the TrimCost object
        %  	>z: optimization vector, used to update the inputs to the
        %  	model
        %  
        %  	<Outputs>
        %  	>xd: steady state derivatives
        %  	>jxd: the jacobian of the steady states
        %  
        %  	<ChangeLog>
        %  	2017-02-07 - Ch. Goettlicher: commenting
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = TrimCost()
        % FALCON.TRIM.CORE.TRIMCOST cost function for trim problems
        %  
        %  	<Description>
        %  	The trim problem is internally transformed into an optimization
        %  	problem of the form
        %       x_dot = f(x,u,p)
        %       x_dot(SteadyState) = f_steady(x,u,p)
        %  
        %     ---                                                   ---
        %     |  min_(x_free,u_free,p_free) f_steady.' * f_steady     |
        %     |     s.t.    y(x,u,p) = y_desired                      |
        %     |             lb < constraints( x,u,p ) < ub            |
        %     ---                                                   ---
        %  
        %   that is, the state derivatives of states that were declared
        %   'SteadyState' is driven to zero by minimizing above cost function,
        %   subject to output and more general constraints.
        %  
        %   This class then implements the necessary cost function computations
        %  
        %  	<ChangeLog>
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function setTrimProblem(obj, TrimProblem)
        % SETTRIMPROBLEM set the trim problem
        %  
        %  	<Syntax>
        %  	setTrimProblem( obj, TrimProblem )
        %  
        %  	<Description>
        %  	set reference to a trim problem handle
        %  
        %  	<Inputs>
        %  	>obj: the HasTrimProblem object
        %  	>TrimProblem: the trim problem
        %  
        %  	<ChangeLog>
        %  	2016-12-15 - Ch. Goettlicher: initial version
        %  	2017-02-07 - Ch. Goettlicher: commenting
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

    end

end