classdef Output < falcon.Output & falcon.trim.core.TVC
    % FALCON.TRIM.OUTPUT output information for trim problems
    %  
    %  	<Description>
    %   container for trim-information on a outputs, i.e. if it is fixed or
    %   free, boundaries, etc.
    %  
    %  	<ChangeLog>
    %  	2017-12-20 - Ch. Goettlicher: final commenting

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % Scaling Parameters initialized with 1. The offset
        % and scaling is assigned in the following way:
        %  
        % x_scaled = (x_unscaled - Offset) * Scaling
        Scaling
        % Offset parameter which is initialized with 0. The offset
        % and scaling is assigned in the following way:
        %  
        % x_scaled = (x_unscaled - Offset) * Scaling
        Offset
        % Name of object.
        Name
        % Lower bound of the falcon.core.OVC value. It is initialized with
        % minus infinity. Scaling and Offset are applied to the
        % LowerBound as well.
        LowerBound
        % Upper bound of the falcon.core.OVC value. It is initialized with
        % plus infinity. Scaling and Offset are applied to the
        % UpperBound as well.
        UpperBound
        % Whether this value is active or not.
        isActive
        % flag indicating if this value is known
        Known
        % the trim value
        TrimValue
        % trim grid
        TrimGrid
    end

    methods
        function setOutputValue(obj, OutputValue)
        % SETOUTPUTVALUE set the current output value
        %  
        %  	<Syntax>
        %  	setOutputValue( obj, OutputValue )
        %  
        %  	<Description>
        %  	set the output value, that has been reached during the
        %  	iteration. Check for proper dimensions before setting the
        %  	value.
        %  
        %  	<Inputs>
        %  	>obj: the Output object
        %  	>OutputValue: current output value
        %  
        %  	<ChangeLog>
        %  	2017-02-07 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = Output(OutputName, varargin)
        % OUTPUT constructor for falcon.trim.Output object
        %  
        %   <Syntax>
        %   obj = Output(OutputName)
        %   obj = Output(OutputName, LowerBound)
        %   obj = Output(OutputName, LowerBound, UpperBound)
        %   obj = Output(OutputName, LowerBound, UpperBound, Scaling)
        %   obj = Output(OutputName, LowerBound, UpperBound, Scaling, Offset)
        %   obj = Output(OutputName, LowerBound, UpperBound, Scaling, Offset, Known)
        %   obj = Output(OutputName, LowerBound, UpperBound, Scaling, Offset, Known, TrimValue)
        %   obj = Output(..., 'Name', Value)
        %  
        %  	<Description>
        %  	create an object, describing the model's output and its
        %  	trim-characteristic. if it is to be forced to its trim value
        %  	(TrimValue), set the Known property to true. Otherwise the
        %  	value will be freely determined during the trim-process.
        %  	Most parameter will be passed to the underlying
        %  	falcon.Control object
        %  
        %  	<Inputs>
        %  	>obj: 
        %  	>varargin: 
        %  	>OutputName: name of for this value
        %  
        %  	<NameValue>
        %  	>LowerBound: lower bound for this values
        %  	>UpperBound: upper bound for this value
        %  	>Scaling: scaling to be used during trimming
        %  	>Offset: offset to be used during trimming (unused so far)
        %  	>Known: flag indicating if this value is to be kept at the
        %  	TrimValue
        %  	>TrimValue: trim value, will be either used as initial guess
        %  	if the value is to be determined, or as target value if it is
        %  	to be kept.
        %  
        %  	<Outputs>
        %  	>obj: the output object
        %  
        %  	<ChangeLog>
        %  	2016-11-22 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = fromModel(ModelHandle)
        % FROMMODEL create output array from model information
        %  
        %   <Syntax>
        %   obj = falcon.trim.Output.fromModel( ModelHandle )
        %  
        %  	<Description>
        %  	create an array of falcon.trim.Output objects from the
        %  	names supplied by the simulation model builder model
        %  	'ModelHandle'. The outputs are all created by supplying
        %  	only the name to the constructor, i.e. all other properties
        %  	are set to their default values (Known = true, lb = -inf,
        %  	ub = +inf, ...).
        %  	
        %  	for the actual creation of parameters, the
        %  	falcon.trim.core.TVC.fromModel_lowLevel method is used.
        %  
        %  	<Inputs>
        %  	>ModelHandle: valid handle to simulation model builder mex
        %  	function
        %  
        %  	<Outputs>
        %  	>obj: array of falcon.trim.Output objects
        %  
        %  	<ChangeLog>
        %  	2017-11-09 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = fromStruct(s)
        % FROMSTRUCT load data from struct
        %  
        %   <Syntax>
        %   obj = falcon.trim.core.Output.fromStruct( s )
        %  
        %  	<Description>
        %  	load TVC data from struct
        %  
        %  	<Inputs>
        %  	>s: struct containing data
        %  
        %  	<Outputs>
        %  	>obj: object containing the output data
        %  
        %  	<ChangeLog>
        %  	2017-03-13 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function valuesStruct = extractValuesStructFrom(obj, source)
        % Extract values struct from Problem or Phase.
        %  
        % <Syntax>
        % valuesStruct = outputArray.extractValuesStructFrom(source)
        %  
        % <Description>
        % Extract the values of the given outputs from a Problem or Phase,
        % returning a struct for name-based access.
        %  
        % <Inputs>
        % > source: a falcon.Problem or falcon.Phase
        %  
        % <Outputs>
        % > valuesStruct: a struct;
        %                 fields: output names,
        %                 values: samples (row vectors)
        end

        function values = extractValuesFrom(obj, source)
        % Extract values array from Problem or Phase.
        %  
        % <Syntax>
        % values = outputArray.extractValuesFrom(source)
        %  
        % <Description>
        % Extract the values of the given outputs from a Problem or Phase.
        %  
        % <Inputs>
        % > source: a falcon.Problem or falcon.Phase
        %  
        % <Outputs>
        % > values: an array of output values;
        %           rows: states, columns: samples
        end

        function strc = ToStruct(obj, varargin)
        % Create a struct from this constraint.
        %  
        % <Syntax>
        % strc = obj.ToStruct()
        %  
        % <Description>
        % Extracts all relevant information from this constraint and stores it
        % in the returned struct.
        %  
        % <NameValue>
        % >DebugData: Setting this option to true enables debug data in
        %   the ToStruct method.
        %  
        % <Keywords>
        % Debugging!Constraint
        end

        function arr = ArrayWith(names, varargin)
        % Create array of falcon.Constraint objects
        %  
        % <Syntax>
        % arr = falcon.Constraint.ArrayWith(names)
        % arr = falcon.Constraint.ArrayWith(names,LowerBound)
        % arr = falcon.Constraint.ArrayWith(names,LowerBound,UpperBound)
        % arr = falcon.Constraint.ArrayWith(names,LowerBound,UpperBound,Scaling)
        % arr = falcon.Constraint.ArrayWith(names,LowerBound,UpperBound,Scaling,Offset)
        % arr = falcon.Constraint.ArrayWith(names,LowerBound,UpperBound,Scaling,Offset,Active)
        %  
        % <Description>
        % Creates an array of falcon.Constraint objects. This method is
        % to a shortcut to create a vector of the constraint objects
        % without creating each object individually.
        %  
        % <Inputs>
        % > names: The names of the falcon.Constraint object as a cell
        % array.
        %  
        % <Optional>
        % > LowerBound: The sorted lower bounds of the constraint object.
        % The size needs to match the number of constraint names.
        % > UpperBound: The sorted upper bounds of the constraint object.
        % The size needs to match the number of constraint names.
        % > Scaling: The sorted scalings of the constraint object.
        % The size needs to match the number of constraint names.
        % > Offset: The sorted offsets of the constraint object.
        % The size needs to match the number of constraint names.
        % > Active: The sorted active flags of the constraint object.
        % The size needs to match the number of constraint names.
        %  
        % <Keywords>
        % Constraint!Array
        end

        function obj = setOffset(obj, offset)
        % Set the offset of this object.
        %  
        % <Syntax>
        % obj.setOffset(offset)
        %  
        % <Description>
        % Set the offset of this object. Please note
        % that the input must be a real, scalar value.
        %  
        % <Inputs>
        % >offset: Offset of this object.
        %  
        % <Keywords>
        % OVC!Offset
        end

        function obj = setScaling(obj, scaling)
        % Set the scaling of this object.
        %  
        % <Syntax>
        % obj.setScaling(scaling)
        %  
        % <Description>
        % The input must be a real positive scalar value and should
        % scale the value of this object to a range between -1 and 1.
        %  
        % <Inputs>
        % >scaling: Scaling of the this object.
        %  
        % <Keywords>
        % OVC!Scaling
        end

        function element = findFirst(self, pattern)
        % Find the first element matching a regex.
        %  
        % <Syntax>
        % element = array.findFirst(pattern)
        %  
        % <Description>
        % Extract the first element from the array whose Name attribute
        % matches the given pattern, as determined by the regexp function.
        % If no element matches, an exception is thrown.
        %  
        % <Inputs>
        % > pattern: see documentation for regexp
        %  
        % <Outputs>
        % > element: the first matching element
        end

        function elements = find(self, pattern)
        % Find elements matching a regex.
        %  
        % <Syntax>
        % elements = array.find(pattern)
        %  
        % <Description>
        % Extract object array elements whose Name attributes match the
        % given pattern, as determined by the regexp function.  The
        % returned array is empty if no element matches.
        %  
        % <Inputs>
        % > pattern: see documentation for regexp
        %  
        % <Outputs>
        % > elements: the entries of the object array whose names match
        end

        function bool = ne(varargin)
        % ~= (NE)   Not equal relation for handles.
        %   Handles are equal if they are handles for the same object and are 
        %   unequal otherwise.
        %  
        %   H1 ~= H2 performs element-wise comparisons between handle arrays H1 
        %   and H2.  H1 and H2 must be of the same dimensions unless one is a 
        %   scalar.  The result is a logical array of the same dimensions, where 
        %   each element is an element-wise equality result.
        %  
        %   If one of H1 or H2 is scalar, scalar expansion is performed and the 
        %   result will match the dimensions of the array that is not scalar.
        %  
        %   TF = NE(H1, H2) stores the result in a logical array of the same
        %   dimensions.
        end

        function bool = eq(varargin)
        % == (EQ)   Test handle equality.
        %   Handles are equal if they are handles for the same object.
        %  
        %   H1 == H2 performs element-wise comparisons between handle arrays H1 and
        %   H2.  H1 and H2 must be of the same dimensions unless one is a scalar.
        %   The result is a logical array of the same dimensions, where each
        %   element is an element-wise equality result.
        %  
        %   If one of H1 or H2 is scalar, scalar expansion is performed and the 
        %   result will match the dimensions of the array that is not scalar.
        %  
        %   TF = EQ(H1, H2) stores the result in a logical array of the same 
        %   dimensions.
        end

        function map = byNameUnique(self)
        % Get a struct for name based access to the object array, allowing
        % only unique names.
        %  
        % <Syntax>
        % [ map ] = obj.byNameUnique()
        %  
        % <Description>
        % The method returns all entries from the object array 'obj' as
        % fields of a struct, with the object Name as field name.  Throws
        % an error in case there are any duplicate names.
        %  
        % <Keywords>
        % OVC!byName!Unique
        end

        function map = byName(self)
        % Get a struct for name based access to the object array.
        %  
        % <Syntax>
        % [ map ] = obj.byName()
        %  
        % <Description>
        % The method returns all entries from the object array 'obj' as
        % fields of a struct, with the object Name as field name.  In case
        % there are duplicate names, the output struct contains object
        % arrays.
        %  
        % <Keywords>
        % OVC!byName
        end

        function self = relaxRange(self, range)
        % Relax the lower and upper bound.
        %  
        % <Syntax>
        % obj.relaxRange(range)
        %  
        % <Description>
        % For current bounds [a, b] and given bounds [c, d], set the bounds
        % to [min(a, c), max(b, d)].
        %  
        % <Inputs>
        % > range: two-element double vector
        end

        function self = relaxBounds(self, lowerBound, upperBound)
        % Relax the lower and upper bound.
        %  
        % <Syntax>
        % obj.relaxBounds(lowerBound, upperBound)
        %  
        % <Description>
        % For current bounds [a, b] and given bounds [c, d], set the bounds
        % to [min(a, c), max(b, d)].
        %  
        % <Inputs>
        % > lowerBound: scalar double
        % > upperBound: scalar double
        end

        function self = tightenRange(self, range)
        % Tighten the lower and upper bound.
        %  
        % <Syntax>
        % obj.tightenRange(range)
        %  
        % <Description>
        % For current bounds [a, b] and given bounds [c, d], set the bounds
        % to [max(a, c), min(b, d)].
        %  
        % <Inputs>
        % > range: two-element double vector
        end

        function self = tightenBounds(self, lowerBound, upperBound)
        % Tighten the lower and upper bound.
        %  
        % <Syntax>
        % obj.tightenBounds(lowerBound, upperBound)
        %  
        % <Description>
        % For current bounds [a, b] and given bounds [c, d], set the bounds
        % to [max(a, c), min(b, d)].
        %  
        % <Inputs>
        % > lowerBound: scalar double
        % > upperBound: scalar double
        end

        function self = setRange(self, range)
        % Set the lower and upper bound using a vector argument.
        %  
        % <Syntax>
        % obj.setRange(range)
        %  
        % <Description>
        % Set the lower and upper bound.
        %  
        % <Inputs>
        % > range: a two-element double vector
        end

        function self = setBounds(self, lowerBound, upperBound)
        % Set the lower and upper bound.
        %  
        % <Syntax>
        % obj.setBounds(lowerBound, upperBound)
        %  
        % <Description>
        % Set the lower and upper bound.
        %  
        % <Inputs>
        % > lowerBound: scalar double
        % > upperBound: scalar double
        end

        function obj = setUpperBound(obj, upperBound)
        % Set the upper bound of this object.
        %  
        % <Syntax>
        % obj.setUpperBound(upperBound)
        %  
        % <Description>
        % Set the upper bound of this object. The input must be a real, scalar
        % value and bigger than the lower bound.
        %  
        % <Inputs>
        % >upperBound: Upper bound of this object.
        %  
        % <Keywords>
        % OVC!Bound!Upper
        end

        function obj = setLowerBound(obj, lowerBound)
        % Set the lower bound of this object.
        %  
        % <Syntax>
        % obj.setLowerBound(lowerBound)
        %  
        % <Description>
        % Set the lower bound of this object. The input must be a real, scalar
        % scalar and smaller than the upper bound.
        %  
        % <Inputs>
        % >lowerBound: Lower bound of this object.
        %  
        % <Keywords>
        % OVC!Bound!Lower
        end

        function setActive(obj, active)
        % Sets the isActive property of this object.
        %  
        % <Syntax>
        % obj.setActive(isActive)
        %  
        % <Description>
        % Set whether this object is active. In case the object is not
        % active it will be ignored during optimization.
        %  
        % <Inputs>
        % isActive: Boolean to check if this object is active or not.
        %  
        % <Keywords>
        % Flags!Active
        end

        function toFile(obj, FileName)
        % TOFILE store data in file
        %  
        %  	<Syntax>
        %  	toFile( obj, FileName )
        %  
        %  	<Description>
        %  	store the TVC data in a struct, which is then stored in a
        %  	file
        %  
        %  	<Inputs>
        %  	>obj: the TVC object
        %  	>FileName: the name of the file, from which the data is to
        %  	be loaded
        %  
        %  	<ChangeLog>
        %  	2017-03-10 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function s = toTrimStruct(obj)
        % TOTRIMSTRUCT Create a struct from this object.
        %  
        %  	<Syntax>
        %  	[ s ] = toTrimStruct( obj )
        %  
        %  	<Description>
        %  	This metod creates a struct of this object including the
        %  	fields of the parent OVC and this TVC properties
        %  
        %  	<Inputs>
        %  	>obj: the TVC object
        %  
        %  	<Outputs>
        %  	>s: struct containing the inherent properties of this object.
        %  
        %  	<ChangeLog>
        %  	2017-03-10 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function value = getGridValue(obj, idx)
        % GETGRIDVALUE get values corresponding to serial idx
        %  
        %  	<Syntax>
        %  	[ value ] = getGridValue( obj, idx )
        %  
        %  	<Description>
        %  	get the values from the grid in serialized fashion; output
        %  	array, if an array of objects is used for the call
        %  
        %  	<Inputs>
        %  	>obj: the TVC object
        %  	>idx: linear indices to extract from the grid
        %  
        %  	<Outputs>
        %  	>value: extracted values
        %  
        %  	<ChangeLog>
        %  	2017-03-10 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function set(obj, name, prop, val)
        % SET certain property by name
        %  
        %  	<Syntax>
        %  	set( obj, name, prop, val )
        %  
        %  	<Description>
        %  	use this method to set a property in an array of TVCs by name
        %  
        %  	<Inputs>
        %  	>obj: 
        %  	>name: name of the TVC to manipulate
        %  	>prop: the name of the property to be set
        %  	>val: the value to be set
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function GridPlot(obj, varargin)
        % plot grid values for this TVC value
        %  
        %   <Syntax>
        %   obj.GridPlot(XValue, 'YValues', YValues)
        %   obj.GridPlot(XValue, 'PlotArguments', PlotArguments)
        %   obj.GridPlot(XValue, 'RemainingBreakpoints', RemainingBreakpoints)
        %  
        %  	<Description>
        %  	plot the values of this TVC over the grid breakpoints. The
        %  	first argument is the x-values over which this TVC is to be
        %  	plotted. The other values have either to be fixed via
        %  	'RemainingBreakpoints' (using the name of the breakpoint as
        %  	Name in the Name-Value pair). Then the closest index in the
        %  	breakpoint vector is chosen. Or, it can be set via 'YValues',
        %  	in which case a three dimensional plot is created, with the
        %  	values of this TVC on the z-axis.
        %  
        %  	<Inputs>
        %  	>obj: 
        %  	>varargin: 
        %  	>XValue: value over which to plot (e.g. altitude, speed,
        %  	etc.)
        %  
        %  	<NameValue>
        %  	>YValues: if this is value is included, a 3D plot is
        %  	generated using XValue, YValues and the grid values on the
        %  	z-axis
        %  	>PlotArguments: cell array of arguments, that are directly
        %  	passed to the underlying plot function
        %  	>RemainingBreakpoints: names of remaining breakpoints to
        %  	fully define a point in the underlying ND-grid
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = fromFile(FileName)
        % FROMFILE load TVC data from file
        %  
        %   <Syntax>
        %   obj = falcon.trim.core.TVC.fromFile( FileName)
        %  
        %  	<Description>
        %  	load TVC data from file
        %  
        %  	<Inputs>
        %  	>FileName: Filename from which to load the data
        %  
        %  	<Outputs>
        %  	>obj: the newly created TVC object
        %  
        %  	<ChangeLog>
        %  	2017-03-13 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

    end

end