classdef TrimProblem < falcon.core.Handle & falcon.core.HasToStruct
    % FALCON.TRIM.TRIMPROBLEM trim problem
    %  
    %  	<Description>
    %  	in this class, all major elements that define a trim problem
    %  	(state, control, parameter configurations, etc.) are assembled
    %  	together, before passing it to a solver to find a solution.
    %  
    %  	<ChangeLog>
    %  	2017-12-20 - Ch. Goettlicher: final commenting

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % options for optimization algorithm
        TrimOptions
        % model function handle
        ModelHandle
        % Model info struct
        ModelInfoStruct
        % handle to additional constraint functions
        ConstraintHandle
        % trim states
        States
        % trim controls
        Controls
        % trim outputs
        Outputs
        % trim parameters
        Parameters
        % additional constraints
        Constraints
        % indices of optimizable states
        OptStateIndices
        % indices of optimizable controls
        OptControlIndices
        % z-scaling
        zScaling
        % f-scaling
        fScaling
        % length of z-vector
        zLength
        % Bounds
        Bounds
        % indices of [x, u] to be trimmed
        OptIdx_total
        % indices of steady states
        x_SteadyStateIdx
        % indices of known outputs
        y_KnownIdx
        % values of known outputs
        y_KnownVal
        % indices of known constraints
        g_KnownIdx
        % values of known constraints
        g_KnownVal
        % number of additional constraints
        nConstr
        % number of states
        nX
        % number of outputs
        nY
        % number of inputs
        nU
        % number of parameters
        nP
    end

    methods
        function toFile(obj, FileName)
        % TOFILE store data in file
        %  
        %  	<Syntax>
        %  	toFile( obj, FileName )
        %  
        %  	<Description>
        %  	store the Trim problemin a struct, which is then stored in
        %  	a file. Can be used to store computed trim-data in a file
        %  
        %  	<Inputs>
        %  	>obj: the TrimProblem object
        %  	>FileName: the target file name
        %  
        %  	<ChangeLog>
        %  	2017-03-14 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function s = ToStruct(obj)
        % TOSTRUCT Create a struct from this object.
        %  
        %  	<Syntax>
        %  	[ s ] = ToStruct( obj )
        %  
        %  	<Description>
        %  	This method creates a struct of this object to be processed
        %  	further or stored to disc.
        %  
        %  	<Inputs>
        %  	>obj: the TrimProblem object
        %  
        %  	<Outputs>
        %  	>s: struct containing the inherent properties of this object.
        %  
        %  	<ChangeLog>
        %  	2017-03-10 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function addTrimConstraint(obj, FunctionHandle, Constraints, varargin)
        % ADDTRIMCONSTRAINT add additional constraints to trim problem
        %  
        %   <Syntax>
        %   obj.addTrimConstraint(Name, FunctionHandle, ConstraintValues)
        %   obj.addTrimConstraint(Name, FunctionHandle, ConstraintValues, CodegenFolder)
        %  
        %  	<Description>
        %  	use point constraint builder to incorporate additional
        %  	non-linear equality constraints on the states, inputs and
        %  	parameters. the FunctionHandle has to accept three arguments,
        %  	first the state vector (as defined in obj.States) secondly
        %  	the control vector (as defined in obj.Controls) and lastly
        %  	the parameter vector (as defined in obj.Parameters).
        %  	The constraints can be any differentiable function
        %  	that evaluates to a coloumn vector.
        %  	The constraints can be defined via falcon.trim.Constraint
        %  	objects. If the Known property is set to true, the problem is
        %  	trimmed so that it is exactly fulfilled. Otherwise, only the
        %  	constraint's boundaries are checked. Also, for Known
        %  	constraints, arrays can be defined to result in a trim grid.
        %  
        %  	<Inputs>
        %  	>obj: the TrimProblem object
        %  	>FunctionHandle: handle to constraint function
        %  	>Constraints: Constraint objects, defining the constraint
        %  	behaviour
        %  
        %  	<NameValue>
        %  	>CodegenFolder: optional code-generation folder for
        %  	artefacts of mex-file generation
        %  
        %  	<ChangeLog>
        %  	2016-11-22 - Ch. Goettlicher: initial version
        %  	2016-12-15 - Ch. Goettlicher: added constraint values
        %  	2017-02-07 - Ch. Goettlicher: commenting
        %  	2017-03-07 - Ch. Goettlicher: added constraint object
        %  	2017-10-31 - Ch. Goettlicher: added Codegen Folder
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function setOutputs(obj, outputs)
        % SETOUTPUTS set outputs for this problem
        %  
        %  	<Syntax>
        %  	setOutputs( obj, outputs )
        %  
        %  	<Description>
        %  	set the outputs (falcon.trim.Output objects), which are to
        %  	be used to trim the model
        %  
        %  	<Inputs>
        %  	>obj: the TrimProblem object
        %  	>outputs: array of output objects used for trimming
        %  
        %  	<ChangeLog>
        %  	2016-11-22 - Ch. Goettlicher: initial version
        %  	2017-02-07 - Ch. Goettlicher: commenting
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function setControls(obj, controls)
        % SETCONTROLS set controls for this problem
        %  
        %  	<Syntax>
        %  	setControls( obj, controls )
        %  
        %  	<Description>
        %  	set the controls (falcon.trim.Control objects), which are to
        %  	be used to trim the model. Match by name and dimension.
        %  
        %  	<Inputs>
        %  	>obj: the TrimProblem object
        %  	>controls: array of control objects used for trimming
        %  
        %  	<ChangeLog>
        %  	2016-11-22 - Ch. Goettlicher: initial version
        %  	2017-02-07 - Ch. Goettlicher: commenting
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function setStates(obj, states)
        % SETSTATES set state for this problem
        %  
        %  	<Syntax>
        %  	setStates( obj, states )
        %  
        %  	<Description>
        %  	set the states (falcon.trim.State objects), which are to be
        %  	used to trim the model. Match by name and dimension.
        %  
        %  	<Inputs>
        %  	>obj: the TrimProblem object
        %  	>states: array of state objects used for trimming
        %  
        %  	<ChangeLog>
        %  	2016-11-22 - Ch. Goettlicher: initial version
        %  	2017-02-07 - Ch. Goettlicher: commenting
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function setParameters(obj, parameters)
        % SETPARAMETERS set parameters for this problem
        %  
        %  	<Syntax>
        %  	setParameters( obj, parameters )
        %  
        %  	<Description>
        %  	set the parameter, which are to be used to trim the model.
        %  	Match by name and dimension.
        %  
        %  	<Inputs>
        %  	>obj: the TrimProblem object
        %  	>parameters: parameter objects used for trimming
        %  
        %  	<ChangeLog>
        %  	2016-11-22 - Ch. Goettlicher: initial version
        %  	2017-02-07 - Ch. Goettlicher: commenting
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = TrimProblem(ModelHandle)
        % TRIMPROBLEM constructor
        %  
        %  	<Syntax>
        %  	[ obj ] = TrimProblem( obj )
        %  
        %  	<Description>
        %  	create a TrimProblem object, by passing a model handle to a
        %  	model created with the falcon.SimulationModelBuilder to
        %  	this constructor
        %  
        %  	<Inputs>
        %  	>ModelHandle: handle to the mex-compilded model, built with
        %  	the falcon.SimulationModelBuilder
        %  
        %  	<Outputs>
        %  	>obj: the newly created TrimProblem object
        %  
        %  	<ChangeLog>
        %   2016-02-23 - M. Rieck: initial version
        %   2016-11-22 - Ch. Goettlicher: update
        %   2016-12-15 - Ch. Goettlicher: separated Problem from solver
        %  	2017-02-07 - Ch. Goettlicher: commenting
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = fromStruct(s)
        % FROMSTRUCT load problem from struct
        %  
        %  	<Syntax>
        %  	[ obj ] = fromStruct( s )
        %  
        %  	<Description>
        %  	load trim problem data from struct
        %  
        %  	<Inputs>
        %  	>s: struct containing data
        %  
        %  	<Outputs>
        %  	>obj: the loaded problem
        %  
        %  	<ChangeLog>
        %  	2017-03-13 - Ch. Goettlicher: initial version 
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = fromFile(FileName)
        % FROMFILE load data from file
        %  
        %  	<Syntax>
        %  	[ obj ] = fromFile( FileName )
        %  
        %  	<Description>
        %  	load trim problem data from file
        %  
        %  	<Inputs>
        %  	>FileName: Filename from which to load the data
        %  
        %  	<Outputs>
        %  	>obj: object containing the trim problem
        %  
        %  	<ChangeLog>
        %  	2017-03-13 - Ch. Goettlicher: initial version 
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

    end

end