% introduction example to using falcon.m in system identification.
% estimate parameters of a second order lag model. Here, the absolute
% minimum syntax is presented.
%
% the artificial measurement data for this example is created using the
% script main_generateDataPt2.m

falcon.init();

if ~exist('./Data/data_Pt2.mat', 'file')
    generateDataPt2();
end

close all
clear
clc

falcon.init();

%% add necessary paths - use relative paths!
BasePath = fileparts(mfilename('fullpath'));
addpath( fullfile(BasePath, 'Model') )
addpath( fullfile(BasePath, 'Data') )

% setup model - i.e. build anew, if it has changed
% buildPt2Model

%% ************************************************************************
% Step:    
% define ID states, controls, outputs, and parameters
% *************************************************************************

% define only parameters, with their initial guess values
IdParameters = [...
    falcon.parest.Parameter('omega_0', 'Value',  2*10, 'Fixed', false);...
    falcon.parest.Parameter(   'zeta', 'Value', 3*0.2, 'Fixed', false);
    falcon.parest.Parameter(      'K', 'Value',     4, 'Fixed', false);
    falcon.parest.Parameter(     'C1', 'Value',   0.5, 'Fixed', true);
    falcon.parest.Parameter(     'C2', 'Value',     1, 'Fixed', true);
    ];

%% ************************************************************************
% Step:    
% load and configure experiments
% *************************************************************************

% use as-is; no configuration is done
experiments = falcon.parest.Experiment.FromFile('data_Pt2.mat');

%% ************************************************************************
% Step:    
% setup estimation problem
% *************************************************************************

% initialize problem
ParameterEstimationProblem = falcon.parest.Problem('Pt2Estimation');
% add model, states, controls, parameters, outputs
ParameterEstimationProblem.addModel( @Pt2, IdParameters );

%% ************************************************************************
% Step:    
% add experiments to problem
% *************************************************************************

ParameterEstimationProblem.addNewExperiment( experiments );
% generate initial guess for states
experiments.makeStateInitialGuess('simulate'); 

%% ************************************************************************
% Step:    
% add cost function to problem
% *************************************************************************

costfunction = falcon.parest.costfunction.MaximumLikelihood();
costfunction.ScalarScalingMode = 'auto_J';  

ParameterEstimationProblem.setCostFunction( costfunction );

%% ************************************************************************
% Step:    
% setup plots
% *************************************************************************

%% ************************************************************************
% Step:    
% choose and configure solver and solve
% *************************************************************************

% ----------------- uncomment for IPOPT solution
optimizer = falcon.solver.ipopt();
ParameterEstimationProblem.Bake()

% % ----------------- uncomment for ML & Gauss-Newton solution
optimizer = falcon.parest.solver(  );

% set problem for solver
optimizer.setProblem(ParameterEstimationProblem);

% solve problem
optimizer.Solve();

%% ************************************************************************
% Step:    
% plot final results
% *************************************************************************

% compute parameter statistics
ParameterEstimationProblem.computeParameterStatistics();

% plot experiments vs. outputs
experiments.plotExperiment();
experiments.plotModelOutput();

% plot parameter statistics
figure('position', [20 700 1500 400])
subplot(1,2,1)
IdParameters.plotRelativeStandardDeviation();
subplot(1,2,2)
IdParameters.plotCorrelationMatrix();

%% ************************************************************************
% Step:    
% store resulting parameters to disc
% *************************************************************************